@@ -2,6 +2,7 @@
 #import "CSFileHandle.h"
 #import "CSMultiHandle.h"
 #import "CSMemoryHandle.h"
+#import "CSStreamHandle.h"
 #import "XADCRCHandle.h"
 
 #import "XADZipParser.h"
@@ -10,6 +11,7 @@
 #import "XAD7ZipParser.h"
 #import "XADStuffItParser.h"
 #import "XADStuffIt5Parser.h"
+#import "XADStuffItSplitParser.h"
 #import "XADStuffItXParser.h"
 #import "XADCompactProParser.h"
 #import "XADDiskDoublerParser.h"
@@ -52,10 +54,27 @@
 
 #include <dirent.h>
 
-NSString *XADIndexKey=@"XADIndex";
 NSString *XADFileNameKey=@"XADFileName";
+NSString *XADCommentKey=@"XADComment";
 NSString *XADFileSizeKey=@"XADFileSize";
 NSString *XADCompressedSizeKey=@"XADCompressedSize";
+NSString *XADCompressionNameKey=@"XADCompressionName";
+
+NSString *XADIsDirectoryKey=@"XADIsDirectory";
+NSString *XADIsResourceForkKey=@"XADIsResourceFork";
+NSString *XADIsArchiveKey=@"XADIsArchive";
+NSString *XADIsHiddenKey=@"XADIsHidden";
+NSString *XADIsLinkKey=@"XADIsLink";
+NSString *XADIsHardLinkKey=@"XADIsHardLink";
+NSString *XADLinkDestinationKey=@"XADLinkDestination";
+NSString *XADIsCharacterDeviceKey=@"XADIsCharacterDevice";
+NSString *XADIsBlockDeviceKey=@"XADIsBlockDevice";
+NSString *XADDeviceMajorKey=@"XADDeviceMajor";
+NSString *XADDeviceMinorKey=@"XADDeviceMinor";
+NSString *XADIsFIFOKey=@"XADIsFIFO";
+NSString *XADIsEncryptedKey=@"XADIsEncrypted";
+NSString *XADIsCorruptedKey=@"XADIsCorrupted";
+
 NSString *XADLastModificationDateKey=@"XADLastModificationDate";
 NSString *XADLastAccessDateKey=@"XADLastAccessDate";
 NSString *XADLastAttributeChangeDateKey=@"XADLastAttributeChangeDate";
@@ -74,27 +93,11 @@ NSString *XADDOSFileAttributesKey=@"XADDOSFileAttributes";
 NSString *XADWindowsFileAttributesKey=@"XADWindowsFileAttributes";
 NSString *XADAmigaProtectionBitsKey=@"XADAmigaProtectionBits";
 
-NSString *XADIsEncryptedKey=@"XADIsEncrypted";
-NSString *XADIsCorruptedKey=@"XADIsCorrupted";
-NSString *XADIsDirectoryKey=@"XADIsDirectory";
-NSString *XADIsResourceForkKey=@"XADIsResourceFork";
-NSString *XADIsArchiveKey=@"XADIsArchive";
-NSString *XADIsHiddenKey=@"XADIsHidden";
-NSString *XADIsLinkKey=@"XADIsLink";
-NSString *XADIsHardLinkKey=@"XADIsHardLink";
-NSString *XADLinkDestinationKey=@"XADLinkDestination";
-NSString *XADIsCharacterDeviceKey=@"XADIsCharacterDevice";
-NSString *XADIsBlockDeviceKey=@"XADIsBlockDevice";
-NSString *XADDeviceMajorKey=@"XADDeviceMajor";
-NSString *XADDeviceMinorKey=@"XADDeviceMinor";
-NSString *XADIsFIFOKey=@"XADIsFIFO";
-
-NSString *XADCommentKey=@"XADComment";
+NSString *XADIndexKey=@"XADIndex";
 NSString *XADDataOffsetKey=@"XADDataOffset";
 NSString *XADDataLengthKey=@"XADDataLength";
 NSString *XADSkipOffsetKey=@"XADSkipOffset";
 NSString *XADSkipLengthKey=@"XADSkipLength";
-NSString *XADCompressionNameKey=@"XADCompressionName";
 
 NSString *XADIsSolidKey=@"XADIsSolid";
 NSString *XADFirstSolidIndexKey=@"XADFirstSolidIndex";
@@ -134,6 +137,7 @@ static int maxheader=0;
 		[XADStuffItParser class],
 		[XADStuffIt5Parser class],
 		[XADStuffIt5ExeParser class],
+		[XADStuffItSplitParser class],
 		[XADStuffItXParser class],
 		[XADBinHexParser class],
 		[XADMacBinaryParser class],
@@ -366,10 +370,15 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 		parsersolidobj=nil;
 		firstsoliddict=prevsoliddict=nil;
 
+		// If the handle is a CSStreamHandle, it can not seek, so treat
+		// this like a solid archive (for instance, .tar.gz). Also, it will
+		// usually be wrapped in a CSSubHandle so unwrap it first.
+		CSHandle *testhandle=handle;
+		if([handle isKindOfClass:[CSSubHandle class]]) testhandle=[(CSSubHandle *)handle parentHandle];
+
+		if([testhandle isKindOfClass:[CSStreamHandle class]]) forcesolid=YES;
+
 		shouldstop=NO;
-
-		autopool=nil;
-
 	}
 	return self;
 }
@@ -443,6 +452,13 @@ name:(NSString *)name propertiesToAdd:(NSMutableDictionary *)props
 {
 	[password autorelease];
 	password=[newpassword retain];
+
+	// Make sure to invalidate any remaining solid handles, as they will need to change
+	// for the new password.
+	[currsolidobj release];
+	currsolidobj=nil;
+	[currsolidhandle release];
+	currsolidhandle=nil;
 }
 
 -(NSString *)encodingName
@@ -756,22 +772,13 @@ regex:(XADRegex *)regex firstFileExtension:(NSString *)firstext
 
 
 
+
 -(void)addEntryWithDictionary:(NSMutableDictionary *)dict
 {
-	[self addEntryWithDictionary:dict retainPosition:NO cyclePools:NO];
+	[self addEntryWithDictionary:dict retainPosition:NO];
 }
 
 -(void)addEntryWithDictionary:(NSMutableDictionary *)dict retainPosition:(BOOL)retainpos
-{
-	[self addEntryWithDictionary:dict retainPosition:retainpos cyclePools:NO];
-}
-
--(void)addEntryWithDictionary:(NSMutableDictionary *)dict cyclePools:(BOOL)cyclepools
-{
-	[self addEntryWithDictionary:dict retainPosition:NO cyclePools:cyclepools];
-}
-
--(void)addEntryWithDictionary:(NSMutableDictionary *)dict retainPosition:(BOOL)retainpos cyclePools:(BOOL)cyclepools
 {
 	// If the caller has requested to stop parsing, discard entry.
 	if(![self shouldKeepParsing]) return;
@@ -805,6 +812,14 @@ regex:(XADRegex *)regex firstFileExtension:(NSString *)firstext
 		case 0xa000: [dict setObject:[NSNumber numberWithBool:YES] forKey:XADIsLinkKey]; break;
 	}
 
+	// Set hidden flag if DOS or Windows file attributes are available and indicate it.
+	NSNumber *attrs=[dict objectForKey:XADDOSFileAttributesKey];
+	if(!attrs) attrs=[dict objectForKey:XADWindowsFileAttributesKey];
+	if(attrs)
+	{
+		if([attrs intValue]&0x02) [dict setObject:[NSNumber numberWithBool:YES] forKey:XADIsHiddenKey];
+	}
+
 	// Extract finderinfo from extended attributes, if present.
 	// Overwrite whatever finderinfo was provided, on the assumption that
 	// the extended attributes are more authoritative.
@@ -846,6 +861,9 @@ regex:(XADRegex *)regex firstFileExtension:(NSString *)firstext
 		if(finderflags) [dict setObject:[NSNumber numberWithInt:finderflags] forKey:XADFinderFlagsKey];
 	}
 
+	// If this is an embedded archive that can't seek, force a solid flag.
+	if(forcesolid) [dict setObject:sourcehandle forKey:XADSolidObjectKey];
+
 	// Handle solidness - set FirstSolid, NextSolid and IsSolid depending on SolidObject.
 	id solidobj=[dict objectForKey:XADSolidObjectKey];
 	if(solidobj)
@@ -897,12 +915,6 @@ regex:(XADRegex *)regex firstFileExtension:(NSString *)firstext
 	else [delegate archiveParser:self foundEntryWithDictionary:dict];
 
 	[delegatepool release];
-
-	if(cyclepools)
-	{
-		[autopool release];
-		autopool=[NSAutoreleasePool new];
-	}
 }
 
 
