@@ -60,30 +60,28 @@
 #define RARMETHOD_GOOD 0x34
 #define RARMETHOD_BEST 0x35
 
-#define RAR_NOSIGNATURE 0
-#define RAR_OLDSIGNATURE 1
-#define RAR_SIGNATURE 2
-
 
 
 static RARBlock ZeroBlock={0};
 
 static inline BOOL IsZeroBlock(RARBlock block) { return block.start==0; }
 
-static int TestSignature(const uint8_t *ptr)
+static BOOL IsRARSignature(const uint8_t *ptr)
 {
-	if(ptr[0]==0x52)
-	if(ptr[1]==0x45&&ptr[2]==0x7e&&ptr[3]==0x5e) return RAR_OLDSIGNATURE;
-	else if(ptr[1]==0x61&&ptr[2]==0x72&&ptr[3]==0x21&&ptr[4]==0x1a&&ptr[5]==0x07&&ptr[6]==0x00) return RAR_SIGNATURE;
+	return ptr[0]==0x52 && ptr[1]==0x61 && ptr[2]==0x72 && ptr[3]==0x21 &&
+	ptr[4]==0x1a && ptr[5]==0x07 && ptr[6]==0x00;
+}
 
-	return RAR_NOSIGNATURE;
+static BOOL IsAncientRARSignature(const uint8_t *ptr)
+{
+	return ptr[0]==0x52 && ptr[1]==0x45 && ptr[2]==0x7e && ptr[3]==0x5e;
 }
 
 static const uint8_t *FindSignature(const uint8_t *ptr,int length)
 {
 	if(length<7) return NULL;
 
-	for(int i=0;i<=length-7;i++) if(TestSignature(&ptr[i])) return &ptr[i];
+	for(int i=0;i<=length-7;i++) if(IsRARSignature(&ptr[i])) return &ptr[i];
 
 	return NULL;
 }
@@ -104,7 +102,8 @@ static const uint8_t *FindSignature(const uint8_t *ptr,int length)
 
 	if(length<7) return NO; // TODO: fix to use correct min size
 
-	if(TestSignature(bytes)) return YES;
+	if(IsRARSignature(bytes)) return YES;
+	if(IsAncientRARSignature(bytes)) return YES;
 
 	return NO;
 }
@@ -163,6 +162,14 @@ static const uint8_t *FindSignature(const uint8_t *ptr,int length)
 	[super dealloc];
 }
 
+-(void)setPassword:(NSString *)newpassword
+{
+	// Make sure to clear key cache if password changes.
+	[keys release];
+	keys=nil;
+	[super setPassword:newpassword];
+}
+
 -(void)parse
 {
 	CSHandle *handle=[self handle];
@@ -170,7 +177,7 @@ static const uint8_t *FindSignature(const uint8_t *ptr,int length)
 	uint8_t buf[7];
 	[handle readBytes:7 toBuffer:buf];	
 
-	if(TestSignature(buf)==RAR_OLDSIGNATURE)
+	if(IsAncientRARSignature(buf))
 	{
 		[self reportInterestingFileWithReason:@"Very old RAR file"];
 		[XADException raiseNotSupportedException];
@@ -199,10 +206,24 @@ static const uint8_t *FindSignature(const uint8_t *ptr,int length)
 			// We hit the end of the file. If we have parts that have no been
 			// emitted yet, do so now, and mark as corrupted as we are missing the
 			// last part.
+
 			if(currparts)
-			[self addEntryWithBlock:&previousblock header:&previousheader
-			compressedSize:totalfilesize files:currfiles solidOffset:totalsolidsize
-			isCorrupted:YES];
+			{
+				// Add current file to solid file list, creating it if necessary.
+				if(!currfiles) currfiles=[NSMutableArray array];
+
+				[currfiles addObject:[NSDictionary dictionaryWithObjectsAndKeys:
+					currparts,@"Parts",
+					[NSNumber numberWithLongLong:previousheader.size],@"OutputLength",
+					[NSNumber numberWithInt:previousheader.version],@"Version",
+					[NSNumber numberWithBool:(block.flags&LHD_PASSWORD)?YES:NO],@"Encrypted",
+					previousheader.salt,@"Salt", // Ends the list if nil.
+				nil]];
+
+				[self addEntryWithBlock:&previousblock header:&previousheader
+				compressedSize:totalfilesize files:currfiles solidOffset:totalsolidsize
+				isCorrupted:YES];
+			}
 
 			break;
 		}
@@ -430,6 +451,7 @@ static const uint8_t *FindSignature(const uint8_t *ptr,int length)
 -(RARBlock)readBlockHeader
 {
 	CSHandle *fh=[self handle];
+	if([fh atEndOfFile]) return ZeroBlock;
 
 	RARBlock block;
 	block.start=[[self handle] offsetInFile];
@@ -442,7 +464,6 @@ static const uint8_t *FindSignature(const uint8_t *ptr,int length)
 
 	block.fh=fh;
 
-	if([fh atEndOfFile]) return ZeroBlock;
 	@try
 	{
 		block.crc=[fh readUInt16LE];
@@ -483,7 +504,11 @@ NSLog(@"%04x %04x %s",~crc&0xffff,block.crc,(~crc&0xffff)==block.crc?"<-------":
 		[fh seekToFileOffset:pos];
 	}*/
 
-	if(block.flags&RARFLAG_LONG_BLOCK) block.datasize=[fh readUInt32LE];
+	// RAR ignores the LONG_BLOCK flag for most chunks. FILE_HEAD, NEWSUB_HEAD,
+	// PROTECT_HEAD, and SUB_HEAD are always treated as long, while most others
+	// are always treated as short. The flag is only used for unknown blocks.
+	// To work around broken archives, we add an exception for FILE_HEAD, at least.
+	if((block.flags&RARFLAG_LONG_BLOCK)||block.type==0x74) block.datasize=[fh readUInt32LE];
 	else block.datasize=0;
 
 	if(archiveflags&MHD_PASSWORD) block.datastart=block.start+((block.headersize+15)&~15)+8;
@@ -644,7 +669,11 @@ isCorrupted:(BOOL)iscorrupted
 	{
 		NSArray *files=[dict objectForKey:XADSolidObjectKey];
 		int index=[[dict objectForKey:@"RARSolidIndex"] intValue];
+
 		handle=[self inputHandleForFileWithIndex:index files:files];
+
+		off_t length=[[dict objectForKey:XADSolidLengthKey] longLongValue];
+		if(length!=[handle fileSize]) handle=[handle nonCopiedSubHandleOfLength:length];
 	}
 	else
 	{
@@ -714,16 +743,17 @@ cryptoVersion:(int)version salt:(NSData *)salt
 		switch(version)
 		{
 			case 13: return [[[XADRAR13CryptHandle alloc] initWithHandle:handle
-			password:[self encodedPassword]] autorelease];
+			length:[handle fileSize] password:[self encodedPassword]] autorelease];
 
 			case 15: return [[[XADRAR15CryptHandle alloc] initWithHandle:handle
-			password:[self encodedPassword]] autorelease];
+			length:[handle fileSize] password:[self encodedPassword]] autorelease];
 
 			case 20: return [[[XADRAR20CryptHandle alloc] initWithHandle:handle
-			password:[self encodedPassword]] autorelease];
+			length:[handle fileSize] password:[self encodedPassword]] autorelease];
 
 			default:
-			return [[[XADRARAESHandle alloc] initWithHandle:handle key:[self keyForSalt:salt]] autorelease];
+			return [[[XADRARAESHandle alloc] initWithHandle:handle
+			length:[handle fileSize] key:[self keyForSalt:salt]] autorelease];
 		}
 	}
 	else return handle;
