@@ -18,7 +18,8 @@ typedef struct StuffItXElement
 {
 	int something,type;
 	int64_t attribs[10];
-	int64_t alglist[4];
+	int64_t alglist[6];
+	int64_t alglist3_extra;
 	off_t dataoffset,actualsize;
 	uint32_t datacrc;
 } StuffItXElement;
@@ -31,7 +32,8 @@ static void DumpElement(StuffItXElement *element);
 static void ReadElement(CSHandle *fh,StuffItXElement *element)
 {
 	for(int i=0;i<10;i++) element->attribs[i]=-1;
-	for(int i=0;i<4;i++) element->alglist[i]=-1;
+	for(int i=0;i<6;i++) element->alglist[i]=-1;
+	element->alglist3_extra=-1;
 
 	element->something=[fh readBitsLE:1];
 	element->type=ReadSitxP2(fh);
@@ -50,9 +52,9 @@ static void ReadElement(CSHandle *fh,StuffItXElement *element)
 		int type=ReadSitxP2(fh);
 		if(type==0) break;
 		uint64_t value=ReadSitxP2(fh);
-		if(type<=4) element->alglist[type-1]=value;
+		if(type<=6) element->alglist[type-1]=value;
 		else NSLog(@"alglist type too big: %d",type);
-		if(type==4) NSLog(@"4 extra: %qu",ReadSitxP2(fh));
+		if(type==4) element->alglist3_extra=ReadSitxP2(fh);
 	}
 
 	element->dataoffset=[fh offsetInFile];
@@ -99,6 +101,9 @@ static CSHandle *HandleForElement(XADStuffItXParser *self,StuffItXElement *eleme
 	int64_t compressionalgorithm=element->alglist[0];
 	int64_t checksumalgorithm=element->alglist[1];
 	int64_t preprocessalgorithm=element->alglist[2];
+	int64_t cryptoalgorithm=element->alglist[3];
+
+	if(cryptoalgorithm>=0) [XADException raiseNotSupportedException];
 
 	[fh seekToFileOffset:element->dataoffset];
 	[fh flushReadBits];
@@ -242,7 +247,8 @@ static void DumpElement(StuffItXElement *element)
 	NSLog(@"(%d) %d: %@",element->something,element->type,name);
 
 	for(int i=0;i<10;i++) if(element->attribs[i]>=0) NSLog(@"       attrib %d: %qu",i,element->attribs[i]);
-	for(int i=0;i<4;i++) if(element->alglist[i]>=0) NSLog(@"       alglist %d: %qu",i,element->alglist[i]);
+	for(int i=0;i<6;i++) if(element->alglist[i]>=0) NSLog(@"       alglist %d: %qu",i,element->alglist[i]);
+	if(element->alglist3_extra>=0) NSLog(@"       alglist 3 extra: %qu",element->alglist3_extra);
 }
 
 
@@ -285,7 +291,15 @@ static void DumpElement(StuffItXElement *element)
 
 	CSHandle *fh=[self handle];
 
-	[fh skipBytes:8];
+	[fh skipBytes:7];
+
+	uint8_t encodingmarker=[fh readUInt8];
+	if(encodingmarker=='?')
+	{
+		// The file has been encoded using a base-N encoder.
+		// TODO: Support these encodings.
+		[XADException raiseNotSupportedException];
+	}
 
 	NSMutableArray *entries=[NSMutableArray array];
 	NSMutableDictionary *entrydict=[NSMutableDictionary dictionary];
@@ -692,7 +706,7 @@ static void DumpElement(StuffItXElement *element)
 				break;
 
 				default:
-					NSLog(@"unknown tag");
+					NSLog(@"unknown tag %d",key);
 					[XADException raiseNotSupportedException];
 				break;
 			}
