@@ -205,6 +205,8 @@ static inline int imin(int a,int b) { return a<b?a:b; }
 	{
 		if(![self shouldKeepParsing]) break;
 
+		NSAutoreleasePool *pool=[NSAutoreleasePool new];
+
 		// Read central directory record.
 		uint32_t centralid=[fh readID];
 		if(centralid!=0x504b0102) [XADException raiseIllegalDataException]; // could try recovering here
@@ -226,12 +228,7 @@ static inline int imin(int a,int b) { return a<b?a:b; }
 		uint32_t extfileattrib=[fh readUInt32LE];
 		off_t locheaderoffset=[fh readUInt32LE];
 
-		[fh skipBytes:namelength+extralength];
-
-		NSData *commentdata=nil;
-		if(commentlength) commentdata=[fh readDataOfLength:commentlength];
-
-		off_t next=[fh offsetInFile];
+		[fh skipBytes:namelength];
 
 		// Read central directory extra fields, just to find the Zip64 field.
 		int length=extralength;
@@ -256,6 +253,12 @@ static inline int imin(int a,int b) { return a<b?a:b; }
 
 			[fh seekToFileOffset:nextextra];
 		}
+		if(length) [fh skipBytes:length];
+
+		NSData *commentdata=nil;
+		if(commentlength) commentdata=[fh readDataOfLength:commentlength];
+        
+		off_t next=[fh offsetInFile];
 
 		// Read local header
 		[fh seekToFileOffset:[self offsetForVolume:startdisk offset:locheaderoffset]];
@@ -298,6 +301,8 @@ static inline int imin(int a,int b) { return a<b?a:b; }
 		else [self setObject:[NSNumber numberWithBool:YES] forPropertyKey:XADIsCorruptedKey];
 
 		[fh seekToFileOffset:next];
+
+		[pool release];
 	}
 }
 
@@ -311,6 +316,8 @@ static inline int imin(int a,int b) { return a<b?a:b; }
 
 	while([self shouldKeepParsing])
 	{
+		NSAutoreleasePool *pool=[NSAutoreleasePool new];
+
 		uint32_t localid;
 		@try { localid=[fh readID]; }
 		@catch(id e) { break; }
@@ -371,6 +378,7 @@ static inline int imin(int a,int b) { return a<b?a:b; }
 			break;
 
 			case 0x504b0102: // central record - stop scanning
+				[pool release];
 				goto end;
 			break;
 
@@ -386,6 +394,8 @@ static inline int imin(int a,int b) { return a<b?a:b; }
 				[self findNextEntry];
 			break;
 		}
+
+		[pool release];
 	}
 
 	end:
@@ -850,21 +860,24 @@ isLastEntry:(BOOL)islastentry
 		}
 		else if(system==3) // Unix
 		{
-			// This is completely unreliable, so don't do it.
-			//int perm=extfileattrib>>16;
-			//[dict setObject:[NSNumber numberWithInt:perm] forKey:XADPosixPermissionsKey];
+			int perm=extfileattrib>>16;
+			// Ignore permissions set to 0, as these are most likely writte by buggy archivers.
+			if(perm!=0) [dict setObject:[NSNumber numberWithInt:perm] forKey:XADPosixPermissionsKey];
 		}
 	}
 
 	#ifdef __APPLE__
 	// Several amazingly broken archivers on OS X create files that do
-	// not contain proper permissions extra records. They still expect apps
-	// and scripts to be executable, though, because Archive Utility by
-	// default makes all files executable. Therefore, create permissions
-	// entries based on the default permissions mask, but only on OS X. 
-	// These will be overridden by extra records right afterwards, if present.
-	mode_t mask=umask(0); umask(mask);
-	[dict setObject:[NSNumber numberWithUnsignedShort:0777&~mask] forKey:XADPosixPermissionsKey];
+	// not contain proper permissions. They still expect apps and scripts
+	// to be executable, though, because Archive Utility by default makes
+	// all files executable. Therefore, for files lacking permissions entries,
+	// make up permissions based on the default mask.
+	// This is only done on OS X.
+	if(![dict objectForKey:XADPosixPermissionsKey])
+	{
+		mode_t mask=umask(0); umask(mask);
+		[dict setObject:[NSNumber numberWithUnsignedShort:0777&~mask] forKey:XADPosixPermissionsKey];
+	}
 	#endif
 
 	if(extradict) [dict addEntriesFromDictionary:extradict];
@@ -881,7 +894,7 @@ isLastEntry:(BOOL)islastentry
 	}
 	else
 	{
-		[self addEntryWithDictionary:dict cyclePools:YES];
+		[self addEntryWithDictionary:dict];
 	}
 }
 
@@ -895,7 +908,7 @@ isLastEntry:(BOOL)islastentry
 
 -(void)addRemeberedEntryAndForget
 {
-	[self addEntryWithDictionary:prevdict cyclePools:NO];
+	[self addEntryWithDictionary:prevdict];
 	[prevdict release];
 	[prevname release];
 	prevdict=nil;
